﻿using System;
using System.Collections.Generic;
using Falcon.StoredProcedureRunner;
using Microsoft.EntityFrameworkCore;
using Microsoft.Extensions.Configuration;

namespace ReportService.Database
{
    /// <summary>
    /// 数据库上下文工厂
    /// </summary>
    public class DbContextFactory
    {
        /// <summary>
        /// 数据库上下文集合
        /// </summary>
        public Dictionary<string,DbContextOptions> List { get; set; } = new();
        /// <summary>
        /// 数据库执行器
        /// </summary>
        public IRunner DbRunner { get; set; }

        /// <summary>
        /// 通过提供配置节点实例化数据库上下文工厂
        /// </summary>
        /// <param name="config">配置节点</param>
        /// <param name="runner">数据库语句执行器</param>
        public DbContextFactory(IConfigurationSection config,IRunner runner) {
            foreach(var item in config.GetChildren()) {
                DbContextOptionsBuilder builder = new();
                builder.UseSqlServer(item.Value);
                this.List.Add(item.Key,builder.Options);
            }
            this.DbRunner = runner;
        }
        /// <summary>
        /// 获取数据库上下文
        /// </summary>
        /// <param name="name">节点定义的数据库名称</param>
        /// <returns>数据上下文</returns>
        public DbContext GetContext(string name) {
            if(List.TryGetValue(name,out var option)) {
                return new DbContext(option);
            }
            throw new Exception("使用的数据库名称错误！");
        }

        /// <summary>
        /// 在指定数据库上执行原始sql语句并返回json格式数据。注意防止sql注入攻击
        /// </summary>
        /// <param name="dbName">数据库名称</param>
        /// <param name="sql">要执行的sql语句</param>
        /// <returns>json格式的数据库返回数据</returns>
        public string RunRawSql(string dbName,string sql) {
            if(this.DbRunner == null) {
                throw new Exception("执行器注册错误，没有实例化IRunner执行器");
            }
            return this.DbRunner.RunRaw(GetContext(dbName),sql);
        }
    }
}
